@@ -1,4 +1,5 @@
 import copy
+import inspect
 from django import template
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
@@ -9,11 +10,14 @@ from importlib import import_module
 register = template.Library()
 
 
-@register.inclusion_tag('admin_shortcuts/base.html')
-def admin_shortcuts():
-    admin_shortcuts = copy.deepcopy(getattr(settings, 'ADMIN_SHORTCUTS', None))
+@register.inclusion_tag('admin_shortcuts/base.html', takes_context=True)
+def admin_shortcuts(context):
+    if 'ADMIN_SHORTCUTS' in context:
+        admin_shortcuts = copy.deepcopy(context['ADMIN_SHORTCUTS'])
+    else:
+        admin_shortcuts = copy.deepcopy(getattr(settings, 'ADMIN_SHORTCUTS', None))
     admin_shortcuts_settings = copy.deepcopy(getattr(settings, 'ADMIN_SHORTCUTS_SETTINGS', None))
-
+    request = context['request']
     if not admin_shortcuts:
         return ''
 
@@ -27,10 +31,11 @@ def admin_shortcuts():
                 except KeyError:
                     raise ImproperlyConfigured(_('settings.ADMIN_SHORTCUTS is improperly configured. '
                                                   'Please supply either a "url" or a "url_name" for each shortcut.'))
+                current_app = shortcut.get('app_name')
                 if isinstance(url_name, list):
-                    shortcut['url'] = reverse(url_name[0], args=url_name[1:])
+                    shortcut['url'] = reverse(url_name[0], args=url_name[1:], current_app=current_app)
                 else:
-                    shortcut['url'] = reverse(url_name)
+                    shortcut['url'] = reverse(url_name, current_app=current_app)
 
                 shortcut['url'] += shortcut.get('url_extra', '')
 
@@ -38,10 +43,10 @@ def admin_shortcuts():
                 shortcut['class'] = get_shortcut_class(shortcut.get('url_name', shortcut['url']))
 
             if shortcut.get('count'):
-                shortcut['count'] = eval_func(shortcut['count'])
+                shortcut['count'] = eval_func(shortcut['count'], request)
 
             if shortcut.get('count_new'):
-                shortcut['count_new'] = eval_func(shortcut['count_new'])
+                shortcut['count_new'] = eval_func(shortcut['count_new'], request)
 
     return {
         'admin_shortcuts': admin_shortcuts,
@@ -64,12 +69,18 @@ def admin_shortcuts_js():
     }
 
 
-def eval_func(func_path):
+def eval_func(func_path, request):
     try:
         module_str = '.'.join(func_path.split('.')[:-1])
         func_str = func_path.split('.')[-1:][0]
         module = import_module(module_str)
-        result = getattr(module, func_str)()
+        result = getattr(module, func_str)
+        if callable(result):
+            args, varargs, keywords, defaults = inspect.getargspec(result)
+            if 'request' in args:
+                result = result(request)
+            else:
+                result = result()
         return result
     except:
         return func_path
